@@ -17,16 +17,13 @@
     Terms <- attr(m, "terms")
     attr(Terms, "intercept") <- 0
     y <- model.response(m)
-    if(!is.null(y)) m <- m[,-1]
+    if(!is.null(y)) m <- m[, -1, drop=FALSE]
     for (i in seq(along=ncol(m))) {
         if(is.ordered(m[[i]])) m[[i]] <- as.numeric(m[[i]])
     }
     ret <- randomForest.default(m, y, ...)
+    ret[["call"]] <- call
     ret$terms <- Terms
-    cl <- match.call()
-    cl[[1]] <- as.name("randomForest")
-    ret$call <- cl
-
     if (!is.null(attr(m, "na.action"))) 
         ret$na.action <- attr(m, "na.action")
     class(ret) <- c("randomForest.formula", "randomForest")
